@@ -2,11 +2,12 @@
 """ Unit tests for parsing Python source into cscope index
 """
 
-import unittest, parser
+import unittest, parser, errno
 try:
     from cStringIO import StringIO
 except ImportError:
     from io import StringIO
+import pycscope
 from pycscope import parseSource, Line, Symbol, NonSymbol, Mark, dumpCst
 
 
@@ -67,6 +68,35 @@ class TestMark(unittest.TestCase):
         m = Mark(Mark.GLOBAL)
         self.assertEqual('\tg', str(m))
 
+    def testRepr(self,):
+        m = Mark(Mark.CLASS)
+        self.assertEqual('<Mark:\\tc>', repr(m))
+
+    def testNotEqual(self,):
+        m = Mark(Mark.FUNC_END)
+        n = Mark(Mark.FUNC_CALL)
+        self.assertTrue(m != n)
+        m = Mark(Mark.FUNC_END)
+        n = Mark(Mark.FUNC_END)
+        self.assertFalse(m != n)
+
+    def testEqual(self,):
+        m = Mark(Mark.FUNC_END)
+        n = Mark(Mark.FUNC_CALL)
+        self.assertFalse(m == n)
+        m = Mark(Mark.FUNC_END)
+        n = Mark(Mark.FUNC_END)
+        self.assertTrue(m == n)
+
+    def testGetattr(self,):
+        m = Mark(Mark.INCLUDE)
+        try:
+            x = m.line_number
+        except AttributeError as e:
+            pass
+        else:
+            self.fail("Expected attribute error referencing non-existent line_number attribute")
+
 
 class TestNonSymbol(unittest.TestCase):
     """ Verify the NonSymbol class.
@@ -97,6 +127,10 @@ class TestNonSymbol(unittest.TestCase):
         ns += NonSymbol("then")
         self.assertEqual("that then", ns.format())
 
+    def testNonSymbol(self,):
+        ns = NonSymbol("bar")
+        self.assertEqual("<NonSymbol:bar>", repr(ns))
+
 
 class TestSymbol(unittest.TestCase):
     """ Verify the Symbol class.
@@ -124,14 +158,14 @@ class TestSymbol(unittest.TestCase):
             failed = True
         except AssertionError:
             failed = False
-        self.assertFalse(failed, "Constructor allowed an empty string without an function end marker")
+        self.assertFalse(failed, "Constructor allowed an empty string without a function end marker")
 
         try:
             s = Symbol('', Mark.FUNC_END)
             failed = False
         except AssertionError:
             failed = True
-        self.assertFalse(failed, "Constructor dis-allowed an empty string with an function end marker")
+        self.assertFalse(failed, "Constructor dis-allowed an empty string with a function end marker")
 
         try:
             s = Symbol(5)
@@ -148,6 +182,28 @@ class TestSymbol(unittest.TestCase):
         s = Symbol('bye', '~')
         self.assertEqual('\t~bye', s.format())
 
+    def testRepr(self,):
+        s = Symbol('foo', '=')
+        self.assertEqual('<Symbol:\t=foo>', repr(s))
+
+    def testCoerce(self,):
+        try:
+            s = Symbol('bar')
+            coerce(1, s)
+        except TypeError as e:
+            pass
+        else:
+            self.fail("Expected a TypeError exception.")
+
+    def testAttributes(self,):
+        s = Symbol('bar')
+        try:
+            x = s.line_number
+        except AttributeError as e:
+            pass
+        else:
+            self.fail("Expected attribute error for referencing non-existent line_number attribute")
+
 
 class TestLine(unittest.TestCase):
     """ Verify the Line class.
@@ -169,14 +225,23 @@ class TestLine(unittest.TestCase):
             failed = True
         except TypeError:
             failed = False
-        self.assertFalse(failed, "Constructor must take one paramter")
+        self.assertFalse(failed, "Constructor must take one parameter")
 
         try:
             l = Line(1,2)
             failed = True
         except TypeError:
             failed = False
-        self.assertFalse(failed, "Constructor takes only one paramter")
+        self.assertFalse(failed, "Constructor takes only one parameter")
+
+    def testGetattr(self,):
+        l = Line(15)
+        try:
+            x = l.line_number
+        except AttributeError as e:
+            pass
+        else:
+            self.fail("Expected an attribute error looking for the non-existent line_number attribute")
 
     def testAddAndRepr(self,):
         l = Line(113)
@@ -193,11 +258,72 @@ class TestLine(unittest.TestCase):
         l += NonSymbol(")")
         self.assertEqual("<Line:117 def \\n\tgx\\n ( \\n\t~y\\n )\\n\\n>", repr(l))
 
+    def testCoerce(self,):
+        try:
+            l = Line(42)
+            coerce(1, l)
+        except TypeError as e:
+            pass
+        else:
+            self.fail("Expected a TypeError exception.")
+
+
+class TestDumpCst(unittest.TestCase):
+
+    def testGoodStream(self,):
+        res = dumpCst(parser.suite("a = 1"), StringIO()).getvalue()
+        exp = "['file_input',\n ['stmt',\n  ['simple_stmt',\n   ['small_stmt',\n    ['expr_stmt',\n     ['testlist',\n      ['test',\n       ['or_test',\n        ['and_test',\n         ['not_test',\n          ['comparison',\n           ['expr',\n            ['xor_expr',\n             ['and_expr',\n              ['shift_expr',\n               ['arith_expr',\n                ['term',\n                 ['factor',\n                  ['power', ['atom', ['NAME', 'a', 1]]]]]]]]]]]]]]]],\n     ['EQUAL', '=', 1],\n     ['testlist',\n      ['test',\n       ['or_test',\n        ['and_test',\n         ['not_test',\n          ['comparison',\n           ['expr',\n            ['xor_expr',\n             ['and_expr',\n              ['shift_expr',\n               ['arith_expr',\n                ['term',\n                 ['factor',\n                  ['power', ['atom', ['NUMBER', '1', 1]]]]]]]]]]]]]]]]]],\n   ['NEWLINE', '', 1]]],\n ['NEWLINE', '', 1],\n ['ENDMARKER', '', 1]]\n"
+        self.assertEquals(res, exp)
+
+    def testGoodStreamBadPipe(self,):
+        import pprint
+        orig_pprint = pprint.pprint
+        def mockEpipe(obj, stm):
+            e = IOError()
+            e.errno = errno.EPIPE
+            raise e
+        pprint.pprint = mockEpipe
+        try:
+            res = dumpCst(parser.suite("a = 1"), StringIO()).getvalue()
+        finally:
+            pprint.pprint = orig_pprint
+        self.assertEquals(res, "")
+
+    def testGoodStreamIOError(self,):
+        import pprint
+        orig_pprint = pprint.pprint
+        def mockEpipe(obj, stm):
+            e = IOError()
+            e.errno = errno.ENOENT
+            raise e
+        pprint.pprint = mockEpipe
+        try:
+            dumpCst(parser.suite("a = 1"), StringIO()).getvalue()
+        except IOError as e:
+            assert e.errno == errno.ENOENT
+        else:
+            self.fail("Expected IOError raised")
+        finally:
+            pprint.pprint = orig_pprint
+
+    def testBadStream(self,):
+        x = 0
+        try:
+            dumpCst(parser.suite("a = 1"), x)
+        except AttributeError as e:
+            pass
+        else:
+            self.fail("Expected a ValueError since we did not give a proper streem")
+
 
 class TestParseSource(unittest.TestCase):
 
     def setUp(self,):
         self.buf = []
+        pycscope.strings_as_symbols = False
+
+    def tearDown(self,):
+        pycscope.strings_as_symbols = False
 
     def verify(self, src, exp, dump=False):
         ''' Run the verification of a source value against an expected output
@@ -241,6 +367,23 @@ class TestParseSource(unittest.TestCase):
         # Verify we can handle a file with just empty lines.
         self.verify(["", ""], [])
 
+    def testDumping(self,):
+        # Verify we can handle dumping
+        self.verify(["", ""], [], True)
+
+    def testMissingNewLine(self,):
+        # Verify we can handle dumping
+        parseSource(" ", self.buf, 0)
+        assert len(self.buf) == 0
+
+    def testSyntaxErrors(self,):
+        try:
+            parseSource("a a (foo)", self.buf, 0)
+        except SyntaxError as e:
+            assert e.lineno == 1
+        else:
+            self.fail("Expected a syntax error")
+
     def testSimpleAssignment(self,):
         # Verify we can handle simple assignment statements.
         self.verify(["a = 4",
@@ -272,7 +415,7 @@ class TestParseSource(unittest.TestCase):
                      "z",
                      " [ { 'a' : ",
                      "x",
-                     " [ ( 1 , ) ] } ] }", 
+                     " [ ( 1 , ) ] } ] }",
                      ""])
 
     def testSimpleTrailerAssignment(self,):
@@ -445,6 +588,63 @@ class TestParseSource(unittest.TestCase):
                      " ] ) = ( ( ( ( 1 , 2 , ) , ( 3 , 4 , 5 ) , ( 6 , ) ) , ( 7 , 8 , ) ) , [ 9 , 10 , 11 ] )",
                      ""])
 
+    def testStringsAsSymbolsOff(self,):
+        self.verify(["foo('abc')"],
+                    ["1 ",
+                     "\t`foo",
+                     " ( 'abc' )",
+                     ""])
+
+    def testStringsAsSymbolsOnSimple(self,):
+        pycscope.strings_as_symbols = True
+        self.verify(["foo('abc')"],
+                    ["1 ",
+                     "\t`foo",
+                     " ( [[ ",
+                     "'abc'",
+                     " ]] )",
+                     ""])
+
+    def testStringsAsSymbolsOnSimpleTriple(self,):
+        pycscope.strings_as_symbols = True
+        self.verify(["foo('''abc''')"],
+                    ["1 ",
+                     "\t`foo",
+                     " ( [[ ",
+                     "'''abc'''",
+                     " ]] )",
+                     ""])
+
+    def testStringsAsSymbolsOnLots(self,):
+        pycscope.strings_as_symbols = True
+        self.verify(["foo('_ABC0123klm_456xzy789XYZ')"],
+                    ["1 ",
+                     "\t`foo",
+                     " ( [[ ",
+                     "'_ABC0123klm_456xzy789XYZ'",
+                     " ]] )",
+                     ""])
+
+    def testStringsAsSymbolsOnSimpleDouble(self,):
+        pycscope.strings_as_symbols = True
+        self.verify(['foo("abc")'],
+                    ["1 ",
+                     "\t`foo",
+                     " ( [[ ",
+                     "\"abc\"",
+                     " ]] )",
+                     ""])
+
+    def testStringsAsSymbolsOnSimpleDoubleTriple(self,):
+        pycscope.strings_as_symbols = True
+        self.verify(['foo("""abc""")'],
+                    ["1 ",
+                     "\t`foo",
+                     " ( [[ ",
+                     "\"\"\"abc\"\"\"",
+                     " ]] )",
+                     ""])
+
     def testNoSymbolForAssignment(self,):
         self.verify(["foo(x,5)[1] = 6"],
                     ["1 ",
@@ -763,18 +963,18 @@ class TestParseSource(unittest.TestCase):
                      "c",
                      ""])
 
-    def testFuncDefNoSymbols(self,):
+    def testFuncDefNoSymbolsInBody(self,):
         # Verify the ability to handle a function definition with no symbols
         # in the body.
         self.verify(["def main():",
                      "\tpass"],
                     ["1 def ",
                      "\t$main",
-                     " ( ) : ",
+                     " ( ) :",
                      "",
                      "2 pass ",
                      "\t}",
-                     ""])
+                     ""], True)
 
     def testFuncDefEndNonSymbol(self,):
         ''' Verify the ability to handle a function definition with symbols in the body, ending with a non-symbol.
@@ -808,8 +1008,8 @@ class TestParseSource(unittest.TestCase):
                      "\t}",
                      ""])
 
-    def testFuncDefNoSymbols(self,):
-        ''' Verify the ability to handle a function definition with no symbols in the body.
+    def testSymbolAssignmentFollowingFuncDef(self,):
+        ''' Verify the ability to handle a function definition end with symbol assignment following.
         '''
         self.verify(["def main():",
                      "\tif x:\t\ty = 0",
